﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;  // COMポート

namespace PetRoboTest
{
    public partial class Mini4LD_Test : Form
    {
        // LED輝度（0～255）
        private byte LLedR, LLedG, LLedB, RLedR, RLedG, RLedB;
        // サーボ角度
        private int[] ServoDeg;
        // サーボ番号
        private const int RF1 = 0;
        private const int RF2 = 1;
        private const int LF1 = 2;
        private const int LF2 = 3;
        private const int RR1 = 4;
        private const int RR2 = 5;
        private const int LR1 = 6;
        private const int LR2 = 7;

        // 初期化
        public Mini4LD_Test()
        {
            InitializeComponent();
            ServoDeg = new int[8];
            for (int i = 0; i < 8; i++)
            {
                ServoDeg[i] = 0;
            }
             LLedR = LLedG = LLedB = RLedR = RLedG = RLedB = 0xFF;
            // COMポートリストの更新
            updateComList();

            // コマンド文字列の表示
            string data = "SA";
            for (int i = 0; i < 8; i++)
            {
                data += (ServoDeg[i] + 90).ToString("X2");
            }
            textBoxPauseCommand.Text = data;
        }

        // COMポートリストの更新
        private void updateComList()
        {
            // コンボボックスのクリア
            comboBoxComNum.Items.Clear();
            // すべてのシリアル・ポート名を取得する
            string[] ports = SerialPort.GetPortNames();
            // 取得したシリアル・ポート名を出力する
            string ComNo = Properties.Settings.Default.ComNo;
            foreach (string port in ports)
            {
                comboBoxComNum.Items.Add(port);
                if(port.Equals(ComNo))
                {
                    comboBoxComNum.SelectedItem = port;
                }
            }
//          comboBoxComNum.SelectedIndex = 0;
        }

        // サーボ角度指令コマンド送信
        private void sendComServo(int servoNo)
        {
            string data;
            data = "S" + servoNo.ToString("X1") + (ServoDeg[servoNo] + 90).ToString("X2");
            sendCommand(data);

            data = "SA";
            for(int i=0;i<8;i++){
                data += (ServoDeg[i] + 90).ToString("X2");
            }
            textBoxPauseCommand.Text = data;
        }

        // コマンド送信
        private void sendCommand(string data)
        {
            string command = "#" + data + "$";

            try
            {
                this.serialPort.Write(command);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /********** 各サーボ角度のスライダ **********/
        private void trackBarRF1_Scroll(object sender, EventArgs e)
        {
            ServoDeg[RF1] = trackBarRF1.Value;
            numericRF1.Value = trackBarRF1.Value;
            sendComServo(RF1);
        }
        private void trackBarRF2_Scroll(object sender, EventArgs e)
        {
            ServoDeg[RF2] = trackBarRF2.Value;
            numericRF2.Value = trackBarRF2.Value;
            sendComServo(RF2);
        }
        private void trackBarLF1_Scroll(object sender, EventArgs e)
        {
            ServoDeg[LF1] = trackBarLF1.Value;
            numericLF1.Value = trackBarLF1.Value;
            sendComServo(LF1);
        }
        private void trackBarLF2_Scroll(object sender, EventArgs e)
        {
            ServoDeg[LF2] = trackBarLF2.Value;
            numericLF2.Value = trackBarLF2.Value;
            sendComServo(LF2);
        }
        private void trackBarRR1_Scroll(object sender, EventArgs e)
        {
            ServoDeg[RR1] = trackBarRR1.Value;
            numericRR1.Value = trackBarRR1.Value;
            sendComServo(RR1);
        }
        private void trackBarRR2_Scroll(object sender, EventArgs e)
        {
            ServoDeg[RR2] = trackBarRR2.Value;
            numericRR2.Value = trackBarRR2.Value;
            sendComServo(RR2);
        }
        private void trackBarLR1_Scroll(object sender, EventArgs e)
        {
            ServoDeg[LR1] = trackBarLR1.Value;
            numericLR1.Value = trackBarLR1.Value;
            sendComServo(LR1);
        }
        private void trackBarLR2_Scroll(object sender, EventArgs e)
        {
            ServoDeg[LR2] = trackBarLR2.Value;
            numericLR2.Value = trackBarLR2.Value;
            sendComServo(LR2);
        }

        /********** 各サーボ角度の数値ボックス **********/
        private void numericRF1_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[RF1] = (int)numericRF1.Value;
            trackBarRF1.Value = (int)numericRF1.Value;
            sendComServo(RF1);
        }
        private void numericRF2_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[RF2] = (int)numericRF2.Value;
            trackBarRF2.Value = (int)numericRF2.Value;
            sendComServo(RF2);
        }
        private void numericLF1_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[LF1] = (int)numericLF1.Value;
            trackBarLF1.Value = (int)numericLF1.Value;
            sendComServo(LF1);
        }
        private void numericLF2_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[LF2] = (int)numericLF2.Value;
            trackBarLF2.Value = (int)numericLF2.Value;
            sendComServo(LF2);
        }
        private void numericRR1_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[RR1] = (int)numericRR1.Value;
            trackBarRR1.Value = (int)numericRR1.Value;
            sendComServo(RR1);
        }
        private void numericRR2_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[RR2] = (int)numericRR2.Value;
            trackBarRR2.Value = (int)numericRR2.Value;
            sendComServo(RR2);
        }
        private void numericLR1_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[LR1] = (int)numericLR1.Value;
            trackBarLR1.Value = (int)numericLR1.Value;
            sendComServo(LR1);
        }
        private void numericLR2_ValueChanged(object sender, EventArgs e)
        {
            ServoDeg[LR2] = (int)numericLR2.Value;
            trackBarLR2.Value = (int)numericLR2.Value;
            sendComServo(LR2);
        }

        /********** 自由文字列の送信ボタン **********/
        private void buttonFree_Click(object sender, EventArgs e)
        {
            string data = textBoxFree.Text;
            sendCommand(data);
        }

        /********** COMポートのリスト更新ボタン **********/
        private void buttonUpdateCom_Click(object sender, EventArgs e)
        {
            updateComList();
        }

        /********** COMポートの開く/閉じるボタン **********/
        private void buttonOpenCom_Click(object sender, EventArgs e)
        {
            try
            {
                if (serialPort.IsOpen)
                {
                    // COMポートを閉じる
                    serialPort.Close();

                    buttonOpenCom.Text = "開く";
                }
                else
                {
                    // COMポートを開く
                    serialPort.PortName = comboBoxComNum.Text;
                    serialPort.BaudRate = 38400;
                    serialPort.Parity = System.IO.Ports.Parity.None;
                    serialPort.DataBits = 8;
                    serialPort.StopBits = System.IO.Ports.StopBits.One;
                    serialPort.Handshake = System.IO.Ports.Handshake.RequestToSend;
                    serialPort.Encoding = Encoding.UTF8;
                    serialPort.NewLine = "\r";
                    serialPort.Open();

                    // 設定の保存
                    Properties.Settings.Default.ComNo = serialPort.PortName;
                    Properties.Settings.Default.Save();

                    buttonOpenCom.Text = "閉じる";
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /********** 各種歩行コマンドの送信ボタン **********/
        private void buttonForward_R_Click(object sender, EventArgs e)
        {
            sendCommand("WRF");
        }
        private void buttonForward_Click(object sender, EventArgs e)
        {
            sendCommand("WNF");
        }
        private void buttonForward_L_Click(object sender, EventArgs e)
        {
            sendCommand("WLF");
        }
        private void buttonTurnRight_Click(object sender, EventArgs e)
        {
            sendCommand("WRN");
        }
        private void buttonStop_Click(object sender, EventArgs e)
        {
            sendCommand("WNN");
        }
        private void buttonTurnLeft_Click(object sender, EventArgs e)
        {
            sendCommand("WLN");
        }
        private void buttonBackward_R_Click(object sender, EventArgs e)
        {
            sendCommand("WRB");
        }
        private void buttonBackward_Click(object sender, EventArgs e)
        {
            sendCommand("WNB");
        }
        private void buttonBackward_L_Click(object sender, EventArgs e)
        {
            sendCommand("WLB");
        }

        // シリアル受信（不使用）
        private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // 受信文字列の取得
            string receivedData;
            try
            {
                receivedData = this.serialPort.ReadExisting();
                receivedData = receivedData.Replace(this.serialPort.NewLine, "\r\n");
            }
            catch (Exception ex)
            {
                receivedData = ex.Message;
            }

            // 受信文字列を追加させる
            AddRecievedDataDelegate add = new AddRecievedDataDelegate(AddRecievedData);
            //hogeTextBox.Invoke(add, receivedData);

        }
        private delegate void AddRecievedDataDelegate(string data);
        private void AddRecievedData(string data)
        {
            //hogeTextBox.Text += data;
        }
    }
}
